@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  BOOSTED MULTIVARIATE TREES FOR LONGITUDINAL DATA (BOOSTMTREE)
-####  Version 1.2.1 (_PROJECT_BUILD_ID_)
+####  Version 1.3.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -49,10 +49,6 @@
 ####    email:  amoljpande@gmail.com
 ####    --------------------------------------------------------------
 ####    Udaya B. Kogalur, Ph.D.
-####    Consultant Staff
-####    Deptartment of Quantitative Health Sciences
-####    Cleveland Clinic Foundation
-####
 ####    Kogalur & Company, Inc.
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
@@ -70,6 +66,8 @@ partialPlot <- function (obj,
                          tm,
                          npts = 25,
                          subset,
+                         conditional.xvars = NULL,
+                         conditional.values = NULL,
                          plot.it = TRUE,
                          ...)
 {
@@ -84,6 +82,19 @@ partialPlot <- function (obj,
     stop("x-variable names provided do not match original variable names")
   }
   n.xvar <- length(xvar.names)
+  if ( !is.null(conditional.xvars) && !is.null(conditional.values) ) {
+    if (length(conditional.xvars) != length(conditional.values)) {
+      stop("conditional x-variable and conditional value vectors are not of same length")
+    }
+    for (i in 1:length(conditional.xvars)) {
+      if( is.factor(obj$x[,conditional.xvars[i] ])){
+      xuniq <- unique(obj$x[,conditional.xvars[i] ])
+      if ( !any(xuniq == conditional.values[i] )  ) {
+        stop("conditional value for the conditional variable:", conditional.xvars[i], " is not from the original data.")
+      }
+     }
+    }
+  }
   tmOrg <- sort(unique(unlist(obj$time)))
   if (missing(tm)) {
     tm.q <- unique(quantile(tmOrg, (1:9)/10, na.rm = TRUE))
@@ -100,6 +111,17 @@ partialPlot <- function (obj,
   if (!missing(subset)) {
     obj$x <- obj$x[subset,, drop = FALSE]
   }
+  if( !is.null(conditional.xvars) && !is.null(conditional.values) ){
+    n.cond.xvar <- length(conditional.xvars)
+    for(i in 1:n.cond.xvar){
+      if(is.factor(obj$x[, conditional.xvars[i]  ])){
+      obj$x[, conditional.xvars[i]  ] <- as.factor(conditional.values[i])
+      }else
+        {
+          obj$x[, conditional.xvars[i]  ] <- conditional.values[i]
+      }
+    }
+  }
   p.obj <- lapply(xvar.names, function(nm) {
     x <- obj$x[, nm]
     n.x <- length(unique(x))
@@ -125,7 +147,7 @@ partialPlot <- function (obj,
   })
   names(l.obj) <- xvar.names
   if (plot.it) {
-    def.par <- par(no.readonly = TRUE) 
+    def.par <- par(no.readonly = TRUE)
     for (k in 1:n.xvar) {
       plot(range(l.obj[[k]][, 1]), range(l.obj[[k]][, -1]), type = "n",
            xlab = xvar.names[k], ylab = "predicted y (adjusted)")
