@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  BOOSTED MULTIVARIATE TREES FOR LONGITUDINAL DATA (BOOSTMTREE)
-####  Version 1.2.1 (_PROJECT_BUILD_ID_)
+####  Version 1.3.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -49,10 +49,6 @@
 ####    email:  amoljpande@gmail.com
 ####    --------------------------------------------------------------
 ####    Udaya B. Kogalur, Ph.D.
-####    Consultant Staff
-####    Deptartment of Quantitative Health Sciences
-####    Cleveland Clinic Foundation
-####
 ####    Kogalur & Company, Inc.
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
@@ -145,6 +141,14 @@ is.hidden.bootstrap <-  function (user.option) {
     as.character(user.option$bootstrap)
   }
 }
+is.hidden.bst.frac <-  function (user.option) {
+  if (is.null(user.option$bst.frac)) {
+    0.632
+  }
+  else {
+    user.option$bst.frac
+  }
+}
 is.hidden.CVlambda <-  function (user.option) {
   if (is.null(user.option$CVlambda)) {
     FALSE
@@ -232,7 +236,7 @@ parse.depth <- function(obj) {
       if (!is.null(obj[[k]])) {
         max(obj[[k]][, "dpthID"], na.rm = TRUE)
       }
-    })), na.rm = TRUE) 
+    })), na.rm = TRUE)
   depth[is.na(depth)] <- treeDepth + 1
   }
   depth
@@ -289,7 +293,7 @@ plot.profile.prx <- function(obj, col = NULL, rnd.case = NULL, cut = .95, restri
                   %*% rnd.prx / sum(rnd.prx))
   prx.which.time <- is.element(time.unq, unlist(lapply(rnd.match, function(i){time[[i]]})))
   prx.time <- time.unq[prx.which.time]
-  prx.mu <- prx.mu[prx.which.time] 
+  prx.mu <- prx.mu[prx.which.time]
   if (is.null(col)) col <- rep(1, length(rnd.prx))
   plot(supsmu(rnd.time, rnd.mean),
        xlim = if (restrictX) range(rnd.time) else range(c(rnd.time, prx.time)),
@@ -326,7 +330,7 @@ rho.inv <- function(ni, rho, tol = 1e-2) {
   }
   else if (rho < 0 && abs(rho + 1 / m) <= tol) {
     (-1 / m + tol) / (m * tol)
-  } 
+  }
   else {
     rho / (1 + m * rho)
   }
@@ -347,3 +351,60 @@ rho.inv.sqrt <- function(ni, rho, tol = 1e-2) {
 sigma.robust <- function(lambda, rho) {
   lambda
 }
+RemoveMiss.Fun <- function(X){
+  n <- nrow(X)
+  WhichRow <- unlist(lapply(1:n,function(i){
+    temp.var <- X[i,]
+    if(all(is.na(temp.var))){
+      out <- "remove"
+    }
+    else{
+      out <- "keep"
+    }
+    out
+  }))
+  WhichRow.remove <- which(WhichRow == "remove")
+  if(length(WhichRow.remove) > 0){
+    X <- X[-WhichRow.remove,]
+  }
+  p <- ncol(X)
+  WhichCol <- unlist(lapply(1:p,function(i){
+    temp.var <- X[,i]
+    if(all(is.na(temp.var))){
+      out <- "remove"
+    }
+    else{
+      out <- "keep"
+    }
+    out
+  }))
+  WhichCol.remove <- which(WhichCol == "remove")
+  if(length(WhichCol.remove) > 0){
+    X <- X[,-WhichCol.remove]
+  }
+  return(list(X = X,id.remove = if (length(WhichRow.remove) > 0) WhichRow.remove else NULL))
+}
+papply <- function(X,FUN,...,mc.preschedule = TRUE, mc.set.seed = TRUE,
+                   mc.silent = FALSE, mc.cores = getOption("mc.cores", 2L),
+                   mc.cleanup = TRUE, mc.allow.recursive = TRUE){
+  result.mclapply <- mclapply(X,FUN,...,mc.preschedule = mc.preschedule, mc.set.seed = mc.set.seed,
+                              mc.silent = mc.silent, mc.cores = mc.cores,
+                              mc.cleanup = mc.cleanup, mc.allow.recursive = mc.allow.recursive)
+  which.null <- which(unlist(lapply(X,function(i){
+    is.null( result.mclapply[[i]] )
+  })))
+  lth.which.null <- length(which.null)
+  if(lth.which.null > 0){
+    result.lapply <- lapply(which.null,FUN,...)
+  }
+  count <- 0
+  result <- lapply(X,function(i){
+    if( any(i == which.null) ){
+      count <<- count + 1
+      result.lapply[[ count ]]
+    }else{
+      result.mclapply[[i]]
+    }
+  })
+return(result)
+}
